/*
 *  Copyright (c) 1996-2007 Digi International Inc., All Rights Reserved
 *
 *  This software contains proprietary and confidential information of Digi
 *  International Inc.  By accepting transfer of this copy, Recipient agrees
 *  to retain this software in confidence, to prevent disclosure to others,
 *  and to make no use of this software other than that for which it was
 *  delivered.  This is an unpublished copyrighted work of Digi International
 *  Inc.  Except as permitted by federal law, 17 USC 117, copying is strictly
 *  prohibited.
 *
 *  Restricted Rights Legend
 *
 *  Use, duplication, or disclosure by the Government is subject to
 *  restrictions set forth in sub-paragraph (c)(1)(ii) of The Rights in
 *  Technical Data and Computer Software clause at DFARS 252.227-7031 or
 *  subparagraphs (c)(1) and (2) of the Commercial Computer Software -
 *  Restricted Rights at 48 CFR 52.227-19, as applicable.
 *
 *  Digi International Inc. 11001 Bren Road East, Minnetonka, MN 55343
 *
 *  This file contains the code that sets up the GPIO ports for the 9750.
 *  This file should not be updated and instead updates should be made to 
 *  gpio.h in the platofrm directory.
 *
 *
 * Edit Date/Ver   Edit Description
 * ==============  =======================================================
 *
 *
 */

/*
 * Do not attempt to trace registers before the C library is
 * setup.  The global variables used by the trace library will
 * be in an undefined state until after the C library has
 * initilized the data segement.
 */
#ifdef NA_REG_TRACE_ENABLED
    #ifdef _REG_DEF_H
        #error Trace code compiled into init code.
    #endif
    #undef NA_REG_TRACE_ENABLED
#endif

#include <stdlib.h>
#include <Npttypes.h>
// #include "bsp.h"
#include <narmled.h>
#include <nareset.h>
#include "init.h"
#include "bootldr.h"
#include "reg_def.h"

#include "gpio_def.h"
#include <gpio.h>
#include "io_ctrl_mod.h"
#include "NA_ISR.h"
#include "errhndlr.h"


#if defined(NS9215)
    #define NA_GPIO_LARGEST_VALID_GPIO_PIN      103
#elif defined(NS9210)
    #define NA_GPIO_LARGEST_VALID_GPIO_PIN      49
#else
    #error Unsupported processor
#endif


#if defined(NS9215)
    #define VALID_GPIO_PIN(channel)     (channel < 108)
#elif defined(NS9210)
/*
 * NS9210 is the same as the NS9215, except it is in a smaller package with
 * fewer pins.  GPIO pins 50-103 don't exist in the NS9210 package.  GPIO
 * pins 104-107 are actually address lines 24 through 27.
 */
    #define VALID_GPIO_PIN(channel)     (   (channel < 50) \
                                         || ((channel > 103) && (channel < 108)) \
                                         ) 
#endif

#define IS_8_WIRE_UART(UART_SETTING)    (UART_SETTING == BSP_GPIO_MUX_SERIAL_8_WIRE_UART)
#define IS_6_WIRE_UART(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_6_WIRE_UART) || IS_8_WIRE_UART(UART_SETTING))
#define IS_4_WIRE_UART(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_4_WIRE_UART) || IS_6_WIRE_UART(UART_SETTING))
#define IS_2_WIRE_UART(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_2_WIRE_UART) || IS_4_WIRE_UART(UART_SETTING))

#define IS_8_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING)    (UART_SETTING == BSP_GPIO_MUX_SERIAL_8_WIRE_UART_USE_ALTERNATE_PATH)
#define IS_6_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_6_WIRE_UART_USE_ALTERNATE_PATH) || IS_8_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING))
#define IS_4_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_4_WIRE_UART_USE_ALTERNATE_PATH) || IS_6_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING))
#define IS_2_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING)    ((UART_SETTING == BSP_GPIO_MUX_SERIAL_2_WIRE_UART_USE_ALTERNATE_PATH) || IS_4_WIRE_UART_USE_ALTERNATE_PATH(UART_SETTING))



#if defined(NS9215)
    #define TIMERS_WITH_NO_PINS      ((BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) \
                                   + ((BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY) && (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_INTERNAL_USE_ONLY)) 

    #if (TIMERS_WITH_NO_PINS < 2)
        #error "NET+OS requires at least 2 timers to be configured without GPIO pins."
        #error "NET+OS uses the NATimerOpen function with the timer number argument set"
        #error "to TIMER_NEXT_AVAILABLE to open the system timer and the timer used by"
        #error "the flash driver and LED driver.  NATimerOpen only opens timers that do"
        #error "not have GPIO pins assigned to them when the timer number is set to"
        #error "TIMER_NEXT_AVAILABLE.  You must configure the GPIO input and output"
        #error "pins for at least two timers to be BSP_GPIO_MUX_INTERNAL_USE_ONLY."
    #endif
#endif







/*
 * Configuration function for GPIO pin 0.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort0(void)
{
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_0_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_0_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 0 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_0_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_0_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_0_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN0 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_0_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_0_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN0 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_0_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_0_VALUE         (1)

    #else

        #define PIN_0_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_0_VALUE         (0)

    #endif
#else
    #define PIN_0_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(0, PIN_0_CONFIG, PIN_0_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 0*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(0, BSP_GPIO_ENABLE_PULLUP_ON_PIN0);
}




/*
 * Configuration function for GPIO pin 1.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort1(void)
{
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 1 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 1 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_1_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 1 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_1_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_1_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_1_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN1 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_1_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_1_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN1 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_1_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_1_VALUE         (1)

    #else

        #define PIN_1_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_1_VALUE         (0)

    #endif
#else
    #define PIN_1_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(1, PIN_1_CONFIG, PIN_1_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 1*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(1, BSP_GPIO_ENABLE_PULLUP_ON_PIN1);
}




/*
 * Configuration function for GPIO pin 2.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort2(void)
{
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 2 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 2 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_2_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 2 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_2_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_2_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_2_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN2 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_2_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_2_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN2 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_2_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_2_VALUE         (1)

    #else

        #define PIN_2_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_2_VALUE         (0)

    #endif
#else
    #define PIN_2_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(2, PIN_2_CONFIG, PIN_2_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 2*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(2, BSP_GPIO_ENABLE_PULLUP_ON_PIN2);
}




/*
 * Configuration function for GPIO pin 3.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort3(void)
{
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_3_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_3_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 3 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_3_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_3_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_3_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN3 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_3_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_3_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN3 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_3_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_3_VALUE         (1)

    #else

        #define PIN_3_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_3_VALUE         (0)

    #endif
#else
    #define PIN_3_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(3, PIN_3_CONFIG, PIN_3_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 3*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(3, BSP_GPIO_ENABLE_PULLUP_ON_PIN3);
}




/*
 * Configuration function for GPIO pin 4.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort4(void)
{
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_4_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_4_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 4 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_4_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_4_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_4_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN4 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_4_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_4_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN4 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_4_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_4_VALUE         (1)

    #else

        #define PIN_4_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_4_VALUE         (0)

    #endif
#else
    #define PIN_4_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(4, PIN_4_CONFIG, PIN_4_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 4*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(4, BSP_GPIO_ENABLE_PULLUP_ON_PIN4);
}




/*
 * Configuration function for GPIO pin 5.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort5(void)
{
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_5_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_5_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 5 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_5_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_5_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_5_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN5 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_5_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_5_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN5 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_5_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_5_VALUE         (1)

    #else

        #define PIN_5_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_5_VALUE         (0)

    #endif
#else
    #define PIN_5_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(5, PIN_5_CONFIG, PIN_5_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 5*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(5, BSP_GPIO_ENABLE_PULLUP_ON_PIN5);
}




/*
 * Configuration function for GPIO pin 6.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort6(void)
{
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 6 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 6 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_6_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 6 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_6_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_6_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_6_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN6 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_6_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_6_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN6 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_6_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_6_VALUE         (1)

    #else

        #define PIN_6_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_6_VALUE         (0)

    #endif
#else
    #define PIN_6_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(6, PIN_6_CONFIG, PIN_6_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 6*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(6, BSP_GPIO_ENABLE_PULLUP_ON_PIN6);
}




/*
 * Configuration function for GPIO pin 7.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort7(void)
{
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_A))
    #define PIN_7_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_7_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 7 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_7_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_7_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_7_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN7 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_7_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_7_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN7 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_7_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_7_VALUE         (1)

    #else

        #define PIN_7_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_7_VALUE         (0)

    #endif
#else
    #define PIN_7_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(7, PIN_7_CONFIG, PIN_7_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 7*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(7, BSP_GPIO_ENABLE_PULLUP_ON_PIN7);
}




/*
 * Configuration function for GPIO pin 8.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort8(void)
{
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_8_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_8_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 8 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_8_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_8_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_8_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN8 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_8_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_8_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN8 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_8_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_8_VALUE         (1)

    #else

        #define PIN_8_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_8_VALUE         (0)

    #endif
#else
    #define PIN_8_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(8, PIN_8_CONFIG, PIN_8_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 8*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(8, BSP_GPIO_ENABLE_PULLUP_ON_PIN8);
}




/*
 * Configuration function for GPIO pin 9.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort9(void)
{
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 9 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 9 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_9_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 9 has been configured for more than one function.
    #error (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_9_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_9_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_9_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN9 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_9_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_9_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN9 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_9_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_9_VALUE         (1)

    #else

        #define PIN_9_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_9_VALUE         (0)

    #endif
#else
    #define PIN_9_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(9, PIN_9_CONFIG, PIN_9_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 9*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(9, BSP_GPIO_ENABLE_PULLUP_ON_PIN9);
}




/*
 * Configuration function for GPIO pin 10.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort10(void)
{
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 10 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 10 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_10_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 10 has been configured for more than one function.
    #error (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_10_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_10_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_10_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN10 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_10_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_10_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN10 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_10_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_10_VALUE         (1)

    #else

        #define PIN_10_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_10_VALUE         (0)

    #endif
#else
    #define PIN_10_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(10, PIN_10_CONFIG, PIN_10_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 10*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(10, BSP_GPIO_ENABLE_PULLUP_ON_PIN10);
}




/*
 * Configuration function for GPIO pin 11.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort11(void)
{
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_11_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_11_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 11 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_11_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_11_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_11_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN11 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_11_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_11_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN11 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_11_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_11_VALUE         (1)

    #else

        #define PIN_11_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_11_VALUE         (0)

    #endif
#else
    #define PIN_11_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(11, PIN_11_CONFIG, PIN_11_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 11*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(11, BSP_GPIO_ENABLE_PULLUP_ON_PIN11);
}




/*
 * Configuration function for GPIO pin 12.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort12(void)
{
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_12_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_12_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 12 has been configured for more than one function.
    #error (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_12_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_12_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_12_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN12 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_12_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_12_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN12 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_12_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_12_VALUE         (1)

    #else

        #define PIN_12_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_12_VALUE         (0)

    #endif
#else
    #define PIN_12_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(12, PIN_12_CONFIG, PIN_12_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 12*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(12, BSP_GPIO_ENABLE_PULLUP_ON_PIN12);
}




/*
 * Configuration function for GPIO pin 13.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort13(void)
{
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_13_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_13_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 13 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_13_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_13_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_13_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN13 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_13_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_13_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN13 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_13_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_13_VALUE         (1)

    #else

        #define PIN_13_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_13_VALUE         (0)

    #endif
#else
    #define PIN_13_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(13, PIN_13_CONFIG, PIN_13_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 13*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(13, BSP_GPIO_ENABLE_PULLUP_ON_PIN13);
}




/*
 * Configuration function for GPIO pin 14.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort14(void)
{
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_14_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_14_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 14 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_14_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_14_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_14_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN14 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_14_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_14_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN14 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_14_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_14_VALUE         (1)

    #else

        #define PIN_14_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_14_VALUE         (0)

    #endif
#else
    #define PIN_14_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(14, PIN_14_CONFIG, PIN_14_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 14*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(14, BSP_GPIO_ENABLE_PULLUP_ON_PIN14);
}




/*
 * Configuration function for GPIO pin 15.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort15(void)
{
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C)) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_C))
    #define PIN_15_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_15_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if ((BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 15 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_15_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if (BSP_GPIO_MUX_SPI_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_15_CONFIG    NA_GPIO_SELECT_MUX_FUNC4
#endif

#if !defined(PIN_15_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN15 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_15_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_15_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN15 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_15_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_15_VALUE         (1)

    #else

        #define PIN_15_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_15_VALUE         (0)

    #endif
#else
    #define PIN_15_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(15, PIN_15_CONFIG, PIN_15_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 15*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(15, BSP_GPIO_ENABLE_PULLUP_ON_PIN15);
}




/*
 * Configuration function for GPIO pin 16.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort16(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 16 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 16 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_16_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 16 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_16_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_16_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_16_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN16 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_16_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_16_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN16 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_16_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_16_VALUE         (1)

    #else

        #define PIN_16_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_16_VALUE         (0)

    #endif
#else
    #define PIN_16_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(16, PIN_16_CONFIG, PIN_16_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 16*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(16, BSP_GPIO_ENABLE_PULLUP_ON_PIN16);
}




/*
 * Configuration function for GPIO pin 17.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort17(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 17 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 17 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_17_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 17 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_17_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_17_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_17_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN17 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_17_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_17_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN17 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_17_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_17_VALUE         (1)

    #else

        #define PIN_17_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_17_VALUE         (0)

    #endif
#else
    #define PIN_17_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(17, PIN_17_CONFIG, PIN_17_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 17*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(17, BSP_GPIO_ENABLE_PULLUP_ON_PIN17);
}




/*
 * Configuration function for GPIO pin 18.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort18(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 18 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 18 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_18_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 18 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_18_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_18_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_18_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN18 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_18_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_18_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN18 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_18_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_18_VALUE         (1)

    #else

        #define PIN_18_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_18_VALUE         (0)

    #endif
#else
    #define PIN_18_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(18, PIN_18_CONFIG, PIN_18_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 18*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(18, BSP_GPIO_ENABLE_PULLUP_ON_PIN18);
}




/*
 * Configuration function for GPIO pin 19.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort19(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 19 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 19 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_19_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 19 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_19_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_19_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_19_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN19 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_19_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_19_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN19 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_19_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_19_VALUE         (1)

    #else

        #define PIN_19_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_19_VALUE         (0)

    #endif
#else
    #define PIN_19_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(19, PIN_19_CONFIG, PIN_19_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 19*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(19, BSP_GPIO_ENABLE_PULLUP_ON_PIN19);
}




/*
 * Configuration function for GPIO pin 20.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort20(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 20 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 20 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_20_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 20 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_20_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_20_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_20_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN20 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_20_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_20_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN20 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_20_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_20_VALUE         (1)

    #else

        #define PIN_20_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_20_VALUE         (0)

    #endif
#else
    #define PIN_20_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(20, PIN_20_CONFIG, PIN_20_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 20*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(20, BSP_GPIO_ENABLE_PULLUP_ON_PIN20);
}




/*
 * Configuration function for GPIO pin 21.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort21(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 21 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 21 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_21_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 21 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_21_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_DMA_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_21_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_21_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN21 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_21_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_21_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN21 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_21_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_21_VALUE         (1)

    #else

        #define PIN_21_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_21_VALUE         (0)

    #endif
#else
    #define PIN_21_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(21, PIN_21_CONFIG, PIN_21_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 21*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(21, BSP_GPIO_ENABLE_PULLUP_ON_PIN21);
}




/*
 * Configuration function for GPIO pin 22.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort22(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 22 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 22 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_22_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 22 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_22_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_DMA_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_22_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_22_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN22 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_22_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_22_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN22 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_22_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_22_VALUE         (1)

    #else

        #define PIN_22_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_22_VALUE         (0)

    #endif
#else
    #define PIN_22_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(22, PIN_22_CONFIG, PIN_22_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 22*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(22, BSP_GPIO_ENABLE_PULLUP_ON_PIN22);
}




/*
 * Configuration function for GPIO pin 23.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort23(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)))
    #error GPIO pin 23 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 23 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_23_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 23 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_23_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_23_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_23_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN23 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_23_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_23_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN23 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_23_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_23_VALUE         (1)

    #else

        #define PIN_23_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_23_VALUE         (0)

    #endif
#else
    #define PIN_23_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(23, PIN_23_CONFIG, PIN_23_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 23*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(23, BSP_GPIO_ENABLE_PULLUP_ON_PIN23);
}




/*
 * Configuration function for GPIO pin 24.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort24(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 24 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 24 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_24_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 24 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_24_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_24_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_24_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_24_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_24_VALUE         (1)

    #else

        #define PIN_24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_24_VALUE         (0)

    #endif
#else
    #define PIN_24_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(24, PIN_24_CONFIG, PIN_24_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 24*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(24, BSP_GPIO_ENABLE_PULLUP_ON_PIN24);
}




/*
 * Configuration function for GPIO pin 25.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort25(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 25 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 25 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_25_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 25 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_25_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_RESET_DONE == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_25_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_25_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_25_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_25_VALUE         (1)

    #else

        #define PIN_25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_25_VALUE         (0)

    #endif
#else
    #define PIN_25_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(25, PIN_25_CONFIG, PIN_25_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 25*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(25, BSP_GPIO_ENABLE_PULLUP_ON_PIN25);
}




/*
 * Configuration function for GPIO pin 26.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort26(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 26 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 26 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_26_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 26 has been configured for more than one function.
    #error (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_26_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_26_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_26_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_26_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_26_VALUE         (1)

    #else

        #define PIN_26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_26_VALUE         (0)

    #endif
#else
    #define PIN_26_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(26, PIN_26_CONFIG, PIN_26_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 26*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(26, BSP_GPIO_ENABLE_PULLUP_ON_PIN26);
}




/*
 * Configuration function for GPIO pin 27.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort27(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 27 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 27 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_27_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 27 has been configured for more than one function.
    #error (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_27_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_27_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_27_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_27_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_27_VALUE         (1)

    #else

        #define PIN_27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_27_VALUE         (0)

    #endif
#else
    #define PIN_27_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(27, PIN_27_CONFIG, PIN_27_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 27*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(27, BSP_GPIO_ENABLE_PULLUP_ON_PIN27);
}




/*
 * Configuration function for GPIO pin 28.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort28(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 28 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 28 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_28_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 28 has been configured for more than one function.
    #error (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_28_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_28_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_28_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN28 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_28_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_28_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN28 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_28_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_28_VALUE         (1)

    #else

        #define PIN_28_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_28_VALUE         (0)

    #endif
#else
    #define PIN_28_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(28, PIN_28_CONFIG, PIN_28_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 28*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(28, BSP_GPIO_ENABLE_PULLUP_ON_PIN28);
}




/*
 * Configuration function for GPIO pin 29.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort29(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 29 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 29 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_29_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 29 has been configured for more than one function.
    #error (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_29_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_29_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_29_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN29 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_29_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_29_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN29 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_29_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_29_VALUE         (1)

    #else

        #define PIN_29_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_29_VALUE         (0)

    #endif
#else
    #define PIN_29_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(29, PIN_29_CONFIG, PIN_29_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 29*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(29, BSP_GPIO_ENABLE_PULLUP_ON_PIN29);
}




/*
 * Configuration function for GPIO pin 30.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort30(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 30 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_30_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (IS_6_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_30_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_30_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN30 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_30_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_30_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN30 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_30_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_30_VALUE         (1)

    #else

        #define PIN_30_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_30_VALUE         (0)

    #endif
#else
    #define PIN_30_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(30, PIN_30_CONFIG, PIN_30_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 30*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(30, BSP_GPIO_ENABLE_PULLUP_ON_PIN30);
}




/*
 * Configuration function for GPIO pin 31.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort31(void)
{
#if ((BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) && (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)))
    #error GPIO pin 31 has been configured for more than one function.
    #error (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32) and (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D)) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_DATA_BUS_WIDTH == BSP_GPIO_MUX_DATA_BUS_WIDTH_32)
    #define PIN_31_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (IS_2_WIRE_UART(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_31_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_31_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN31 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_31_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_31_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN31 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_31_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_31_VALUE         (1)

    #else

        #define PIN_31_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_31_VALUE         (0)

    #endif
#else
    #define PIN_31_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(31, PIN_31_CONFIG, PIN_31_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 31*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(31, BSP_GPIO_ENABLE_PULLUP_ON_PIN31);
}




/*
 * Configuration function for GPIO pin 32.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort32(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 32 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_32_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_32_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_32_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN32 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_32_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_32_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN32 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_32_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_32_VALUE         (1)

    #else

        #define PIN_32_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_32_VALUE         (0)

    #endif
#else
    #define PIN_32_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(32, PIN_32_CONFIG, PIN_32_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 32*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(32, BSP_GPIO_ENABLE_PULLUP_ON_PIN32);
}




/*
 * Configuration function for GPIO pin 33.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort33(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 33 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_33_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_33_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_33_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN33 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_33_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_33_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN33 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_33_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_33_VALUE         (1)

    #else

        #define PIN_33_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_33_VALUE         (0)

    #endif
#else
    #define PIN_33_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(33, PIN_33_CONFIG, PIN_33_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 33*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(33, BSP_GPIO_ENABLE_PULLUP_ON_PIN33);
}




/*
 * Configuration function for GPIO pin 34.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort34(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 34 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_34_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_34_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_34_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN34 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_34_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_34_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN34 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_34_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_34_VALUE         (1)

    #else

        #define PIN_34_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_34_VALUE         (0)

    #endif
#else
    #define PIN_34_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(34, PIN_34_CONFIG, PIN_34_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 34*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(34, BSP_GPIO_ENABLE_PULLUP_ON_PIN34);
}




/*
 * Configuration function for GPIO pin 35.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort35(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 35 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_35_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_35_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_35_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN35 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_35_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_35_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN35 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_35_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_35_VALUE         (1)

    #else

        #define PIN_35_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_35_VALUE         (0)

    #endif
#else
    #define PIN_35_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(35, PIN_35_CONFIG, PIN_35_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 35*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(35, BSP_GPIO_ENABLE_PULLUP_ON_PIN35);
}




/*
 * Configuration function for GPIO pin 36.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort36(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 36 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_36_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_36_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_36_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN36 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_36_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_36_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN36 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_36_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_36_VALUE         (1)

    #else

        #define PIN_36_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_36_VALUE         (0)

    #endif
#else
    #define PIN_36_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(36, PIN_36_CONFIG, PIN_36_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 36*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(36, BSP_GPIO_ENABLE_PULLUP_ON_PIN36);
}




/*
 * Configuration function for GPIO pin 37.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort37(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 37 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_37_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_37_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_37_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN37 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_37_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_37_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN37 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_37_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_37_VALUE         (1)

    #else

        #define PIN_37_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_37_VALUE         (0)

    #endif
#else
    #define PIN_37_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(37, PIN_37_CONFIG, PIN_37_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 37*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(37, BSP_GPIO_ENABLE_PULLUP_ON_PIN37);
}




/*
 * Configuration function for GPIO pin 38.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort38(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 38 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_38_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_38_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_38_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN38 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_38_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_38_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN38 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_38_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_38_VALUE         (1)

    #else

        #define PIN_38_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_38_VALUE         (0)

    #endif
#else
    #define PIN_38_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(38, PIN_38_CONFIG, PIN_38_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 38*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(38, BSP_GPIO_ENABLE_PULLUP_ON_PIN38);
}




/*
 * Configuration function for GPIO pin 39.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort39(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 39 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_39_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_39_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_39_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN39 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_39_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_39_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN39 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_39_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_39_VALUE         (1)

    #else

        #define PIN_39_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_39_VALUE         (0)

    #endif
#else
    #define PIN_39_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(39, PIN_39_CONFIG, PIN_39_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 39*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(39, BSP_GPIO_ENABLE_PULLUP_ON_PIN39);
}




/*
 * Configuration function for GPIO pin 40.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort40(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 40 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_40_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_40_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_40_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN40 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_40_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_40_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN40 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_40_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_40_VALUE         (1)

    #else

        #define PIN_40_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_40_VALUE         (0)

    #endif
#else
    #define PIN_40_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(40, PIN_40_CONFIG, PIN_40_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 40*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(40, BSP_GPIO_ENABLE_PULLUP_ON_PIN40);
}




/*
 * Configuration function for GPIO pin 41.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort41(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 41 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_41_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_41_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_41_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN41 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_41_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_41_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN41 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_41_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_41_VALUE         (1)

    #else

        #define PIN_41_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_41_VALUE         (0)

    #endif
#else
    #define PIN_41_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(41, PIN_41_CONFIG, PIN_41_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 41*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(41, BSP_GPIO_ENABLE_PULLUP_ON_PIN41);
}




/*
 * Configuration function for GPIO pin 42.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort42(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 42 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_42_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_42_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_42_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN42 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_42_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_42_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN42 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_42_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_42_VALUE         (1)

    #else

        #define PIN_42_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_42_VALUE         (0)

    #endif
#else
    #define PIN_42_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(42, PIN_42_CONFIG, PIN_42_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 42*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(42, BSP_GPIO_ENABLE_PULLUP_ON_PIN42);
}




/*
 * Configuration function for GPIO pin 43.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort43(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 43 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_43_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_43_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_43_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN43 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_43_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_43_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN43 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_43_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_43_VALUE         (1)

    #else

        #define PIN_43_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_43_VALUE         (0)

    #endif
#else
    #define PIN_43_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(43, PIN_43_CONFIG, PIN_43_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 43*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(43, BSP_GPIO_ENABLE_PULLUP_ON_PIN43);
}




/*
 * Configuration function for GPIO pin 44.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort44(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 44 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_44_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_44_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_44_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN44 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_44_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_44_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN44 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_44_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_44_VALUE         (1)

    #else

        #define PIN_44_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_44_VALUE         (0)

    #endif
#else
    #define PIN_44_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(44, PIN_44_CONFIG, PIN_44_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 44*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(44, BSP_GPIO_ENABLE_PULLUP_ON_PIN44);
}




/*
 * Configuration function for GPIO pin 45.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort45(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 45 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_45_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_45_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_45_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN45 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_45_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_45_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN45 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_45_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_45_VALUE         (1)

    #else

        #define PIN_45_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_45_VALUE         (0)

    #endif
#else
    #define PIN_45_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(45, PIN_45_CONFIG, PIN_45_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 45*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(45, BSP_GPIO_ENABLE_PULLUP_ON_PIN45);
}




/*
 * Configuration function for GPIO pin 46.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort46(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 46 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_46_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_46_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_46_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN46 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_46_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_46_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN46 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_46_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_46_VALUE         (1)

    #else

        #define PIN_46_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_46_VALUE         (0)

    #endif
#else
    #define PIN_46_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(46, PIN_46_CONFIG, PIN_46_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 46*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(46, BSP_GPIO_ENABLE_PULLUP_ON_PIN46);
}




/*
 * Configuration function for GPIO pin 47.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort47(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 47 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_47_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_47_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if !defined(PIN_47_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN47 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_47_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_47_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN47 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_47_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_47_VALUE         (1)

    #else

        #define PIN_47_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_47_VALUE         (0)

    #endif
#else
    #define PIN_47_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(47, PIN_47_CONFIG, PIN_47_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 47*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(47, BSP_GPIO_ENABLE_PULLUP_ON_PIN47);
}




/*
 * Configuration function for GPIO pin 48.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort48(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_48_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if !defined(PIN_48_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN48 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_48_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_48_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN48 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_48_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_48_VALUE         (1)

    #else

        #define PIN_48_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_48_VALUE         (0)

    #endif
#else
    #define PIN_48_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(48, PIN_48_CONFIG, PIN_48_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 48*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(48, BSP_GPIO_ENABLE_PULLUP_ON_PIN48);
}




/*
 * Configuration function for GPIO pin 49.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort49(void)
{
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_49_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if !defined(PIN_49_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN49 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_49_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_49_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN49 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_49_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_49_VALUE         (1)

    #else

        #define PIN_49_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_49_VALUE         (0)

    #endif
#else
    #define PIN_49_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(49, PIN_49_CONFIG, PIN_49_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 49*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(49, BSP_GPIO_ENABLE_PULLUP_ON_PIN49);
}


#if NS9215


/*
 * Configuration function for GPIO pin 50.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort50(void)
{
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 50 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) && (BSP_GPIO_MUX_IOP_1_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 50 has been configured for more than one function.
    #error (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII) and (BSP_GPIO_MUX_IOP_1_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_ETHERNET_PHY_INTERFACE == BSP_GPIO_MUX_ETHERNET_MII)
    #define PIN_50_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 50 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_50_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_50_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_50_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN50 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_50_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_50_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN50 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_50_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_50_VALUE         (1)

    #else

        #define PIN_50_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_50_VALUE         (0)

    #endif
#else
    #define PIN_50_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(50, PIN_50_CONFIG, PIN_50_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 50*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(50, BSP_GPIO_ENABLE_PULLUP_ON_PIN50);
}




/*
 * Configuration function for GPIO pin 51.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort51(void)
{
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 51 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 51 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_51_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 51 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_51_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_51_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_51_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN51 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_51_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_51_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN51 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_51_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_51_VALUE         (1)

    #else

        #define PIN_51_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_51_VALUE         (0)

    #endif
#else
    #define PIN_51_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(51, PIN_51_CONFIG, PIN_51_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 51*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(51, BSP_GPIO_ENABLE_PULLUP_ON_PIN51);
}




/*
 * Configuration function for GPIO pin 52.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort52(void)
{
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 52 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 52 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_52_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 52 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_52_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_52_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_52_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN52 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_52_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_52_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN52 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_52_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_52_VALUE         (1)

    #else

        #define PIN_52_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_52_VALUE         (0)

    #endif
#else
    #define PIN_52_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(52, PIN_52_CONFIG, PIN_52_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 52*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(52, BSP_GPIO_ENABLE_PULLUP_ON_PIN52);
}




/*
 * Configuration function for GPIO pin 53.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort53(void)
{
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 53 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 53 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_53_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 53 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_53_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_53_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_53_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN53 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_53_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_53_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN53 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_53_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_53_VALUE         (1)

    #else

        #define PIN_53_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_53_VALUE         (0)

    #endif
#else
    #define PIN_53_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(53, PIN_53_CONFIG, PIN_53_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 53*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(53, BSP_GPIO_ENABLE_PULLUP_ON_PIN53);
}




/*
 * Configuration function for GPIO pin 54.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort54(void)
{
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 54 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 54 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_54_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 54 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_54_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_54_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_54_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN54 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_54_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_54_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN54 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_54_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_54_VALUE         (1)

    #else

        #define PIN_54_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_54_VALUE         (0)

    #endif
#else
    #define PIN_54_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(54, PIN_54_CONFIG, PIN_54_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 54*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(54, BSP_GPIO_ENABLE_PULLUP_ON_PIN54);
}




/*
 * Configuration function for GPIO pin 55.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort55(void)
{
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 55 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 55 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_55_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 55 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_55_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_55_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_55_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN55 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_55_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_55_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN55 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_55_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_55_VALUE         (1)

    #else

        #define PIN_55_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_55_VALUE         (0)

    #endif
#else
    #define PIN_55_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(55, PIN_55_CONFIG, PIN_55_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 55*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(55, BSP_GPIO_ENABLE_PULLUP_ON_PIN55);
}




/*
 * Configuration function for GPIO pin 56.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort56(void)
{
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 56 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 56 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_56_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 56 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_56_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_56_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_56_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN56 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_56_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_56_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN56 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_56_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_56_VALUE         (1)

    #else

        #define PIN_56_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_56_VALUE         (0)

    #endif
#else
    #define PIN_56_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(56, PIN_56_CONFIG, PIN_56_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 56*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(56, BSP_GPIO_ENABLE_PULLUP_ON_PIN56);
}




/*
 * Configuration function for GPIO pin 57.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort57(void)
{
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 57 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 57 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_57_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 57 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_57_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_57_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_57_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN57 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_57_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_57_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN57 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_57_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_57_VALUE         (1)

    #else

        #define PIN_57_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_57_VALUE         (0)

    #endif
#else
    #define PIN_57_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(57, PIN_57_CONFIG, PIN_57_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 57*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(57, BSP_GPIO_ENABLE_PULLUP_ON_PIN57);
}




/*
 * Configuration function for GPIO pin 58.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort58(void)
{
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 58 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 58 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_B))
    #define PIN_58_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 58 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_58_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_58_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_58_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN58 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_58_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_58_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN58 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_58_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_58_VALUE         (1)

    #else

        #define PIN_58_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_58_VALUE         (0)

    #endif
#else
    #define PIN_58_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(58, PIN_58_CONFIG, PIN_58_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 58*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(58, BSP_GPIO_ENABLE_PULLUP_ON_PIN58);
}




/*
 * Configuration function for GPIO pin 59.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort59(void)
{
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 59 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 59 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_59_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 59 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_59_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_59_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_59_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN59 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_59_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_59_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN59 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_59_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_59_VALUE         (1)

    #else

        #define PIN_59_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_59_VALUE         (0)

    #endif
#else
    #define PIN_59_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(59, PIN_59_CONFIG, PIN_59_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 59*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(59, BSP_GPIO_ENABLE_PULLUP_ON_PIN59);
}




/*
 * Configuration function for GPIO pin 60.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort60(void)
{
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 60 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 60 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_60_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 60 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_60_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_60_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_60_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN60 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_60_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_60_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN60 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_60_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_60_VALUE         (1)

    #else

        #define PIN_60_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_60_VALUE         (0)

    #endif
#else
    #define PIN_60_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(60, PIN_60_CONFIG, PIN_60_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 60*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(60, BSP_GPIO_ENABLE_PULLUP_ON_PIN60);
}




/*
 * Configuration function for GPIO pin 61.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort61(void)
{
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 61 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 61 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_61_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 61 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_61_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_61_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_61_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN61 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_61_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_61_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN61 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_61_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_61_VALUE         (1)

    #else

        #define PIN_61_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_61_VALUE         (0)

    #endif
#else
    #define PIN_61_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(61, PIN_61_CONFIG, PIN_61_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 61*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(61, BSP_GPIO_ENABLE_PULLUP_ON_PIN61);
}




/*
 * Configuration function for GPIO pin 62.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort62(void)
{
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 62 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 62 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_62_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 62 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_62_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_62_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_62_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN62 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_62_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_62_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN62 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_62_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_62_VALUE         (1)

    #else

        #define PIN_62_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_62_VALUE         (0)

    #endif
#else
    #define PIN_62_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(62, PIN_62_CONFIG, PIN_62_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 62*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(62, BSP_GPIO_ENABLE_PULLUP_ON_PIN62);
}




/*
 * Configuration function for GPIO pin 63.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort63(void)
{
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 63 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 63 has been configured for more than one function.
    #error (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_8_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_63_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 63 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_63_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_63_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_63_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN63 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_63_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_63_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN63 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_63_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_63_VALUE         (1)

    #else

        #define PIN_63_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_63_VALUE         (0)

    #endif
#else
    #define PIN_63_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(63, PIN_63_CONFIG, PIN_63_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 63*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(63, BSP_GPIO_ENABLE_PULLUP_ON_PIN63);
}




/*
 * Configuration function for GPIO pin 64.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort64(void)
{
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 64 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 64 has been configured for more than one function.
    #error (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_4_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_64_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 64 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_64_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_64_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_64_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN64 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_64_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_64_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN64 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_64_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_64_VALUE         (1)

    #else

        #define PIN_64_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_64_VALUE         (0)

    #endif
#else
    #define PIN_64_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(64, PIN_64_CONFIG, PIN_64_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 64*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(64, BSP_GPIO_ENABLE_PULLUP_ON_PIN64);
}




/*
 * Configuration function for GPIO pin 65.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort65(void)
{
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 65 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 65 has been configured for more than one function.
    #error (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_6_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_65_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 65 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_65_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_65_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_65_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN65 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_65_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_65_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN65 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_65_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_65_VALUE         (1)

    #else

        #define PIN_65_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_65_VALUE         (0)

    #endif
#else
    #define PIN_65_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(65, PIN_65_CONFIG, PIN_65_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 65*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(65, BSP_GPIO_ENABLE_PULLUP_ON_PIN65);
}




/*
 * Configuration function for GPIO pin 66.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort66(void)
{
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 66 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 66 has been configured for more than one function.
    #error (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D)) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (IS_2_WIRE_UART_USE_ALTERNATE_PATH(BSP_GPIO_MUX_SERIAL_D))
    #define PIN_66_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 66 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_66_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_66_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_66_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN66 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_66_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_66_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN66 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_66_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_66_VALUE         (1)

    #else

        #define PIN_66_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_66_VALUE         (0)

    #endif
#else
    #define PIN_66_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(66, PIN_66_CONFIG, PIN_66_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 66*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(66, BSP_GPIO_ENABLE_PULLUP_ON_PIN66);
}




/*
 * Configuration function for GPIO pin 67.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort67(void)
{
#if ((BSP_GPIO_MUX_IOP_O_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 67 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_O_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_O_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 67 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_O_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_O_INPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_67_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_0_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 67 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_OUTPUT_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_67_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_67_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_67_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN67 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_67_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_67_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN67 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_67_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_67_VALUE         (1)

    #else

        #define PIN_67_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_67_VALUE         (0)

    #endif
#else
    #define PIN_67_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(67, PIN_67_CONFIG, PIN_67_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 67*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(67, BSP_GPIO_ENABLE_PULLUP_ON_PIN67);
}




/*
 * Configuration function for GPIO pin 68.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort68(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 68 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 68 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_68_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 68 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_0 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_68_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_68_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_68_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN68 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_68_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_68_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN68 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_68_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_68_VALUE         (1)

    #else

        #define PIN_68_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_68_VALUE         (0)

    #endif
#else
    #define PIN_68_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(68, PIN_68_CONFIG, PIN_68_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 68*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(68, BSP_GPIO_ENABLE_PULLUP_ON_PIN68);
}




/*
 * Configuration function for GPIO pin 69.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort69(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 69 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 69 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_69_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 69 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_1 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_69_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_69_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_69_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN69 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_69_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_69_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN69 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_69_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_69_VALUE         (1)

    #else

        #define PIN_69_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_69_VALUE         (0)

    #endif
#else
    #define PIN_69_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(69, PIN_69_CONFIG, PIN_69_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 69*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(69, BSP_GPIO_ENABLE_PULLUP_ON_PIN69);
}




/*
 * Configuration function for GPIO pin 70.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort70(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 70 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_PWM_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 70 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_PWM_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_70_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_PWM_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 70 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_PWM_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_2 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_70_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_PWM_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_70_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_70_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN70 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_70_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_70_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN70 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_70_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_70_VALUE         (1)

    #else

        #define PIN_70_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_70_VALUE         (0)

    #endif
#else
    #define PIN_70_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(70, PIN_70_CONFIG, PIN_70_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 70*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(70, BSP_GPIO_ENABLE_PULLUP_ON_PIN70);
}




/*
 * Configuration function for GPIO pin 71.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort71(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 71 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_PWM_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 71 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_PWM_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_71_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) && (BSP_GPIO_MUX_PWM_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 71 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) and (BSP_GPIO_MUX_PWM_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_3 == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_71_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_PWM_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_71_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_71_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN71 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_71_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_71_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN71 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_71_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_71_VALUE         (1)

    #else

        #define PIN_71_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_71_VALUE         (0)

    #endif
#else
    #define PIN_71_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(71, PIN_71_CONFIG, PIN_71_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 71*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(71, BSP_GPIO_ENABLE_PULLUP_ON_PIN71);
}




/*
 * Configuration function for GPIO pin 72.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort72(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 72 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 72 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_72_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 72 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_72_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_PWM_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_72_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_72_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN72 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_72_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_72_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN72 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_72_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_72_VALUE         (1)

    #else

        #define PIN_72_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_72_VALUE         (0)

    #endif
#else
    #define PIN_72_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(72, PIN_72_CONFIG, PIN_72_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 72*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(72, BSP_GPIO_ENABLE_PULLUP_ON_PIN72);
}




/*
 * Configuration function for GPIO pin 73.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort73(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 73 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 73 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_73_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 73 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_5 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_73_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_PWM_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_73_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_73_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN73 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_73_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_73_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN73 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_73_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_73_VALUE         (1)

    #else

        #define PIN_73_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_73_VALUE         (0)

    #endif
#else
    #define PIN_73_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(73, PIN_73_CONFIG, PIN_73_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 73*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(73, BSP_GPIO_ENABLE_PULLUP_ON_PIN73);
}




/*
 * Configuration function for GPIO pin 74.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort74(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 74 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 74 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_74_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 74 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_6 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_74_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_0_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_74_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_74_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN74 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_74_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_74_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN74 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_74_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_74_VALUE         (1)

    #else

        #define PIN_74_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_74_VALUE         (0)

    #endif
#else
    #define PIN_74_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(74, PIN_74_CONFIG, PIN_74_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 74*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(74, BSP_GPIO_ENABLE_PULLUP_ON_PIN74);
}




/*
 * Configuration function for GPIO pin 75.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort75(void)
{
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 75 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 75 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_75_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 75 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_GEN_IO_7 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_75_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_1_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_75_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_75_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN75 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_75_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_75_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN75 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_75_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_75_VALUE         (1)

    #else

        #define PIN_75_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_75_VALUE         (0)

    #endif
#else
    #define PIN_75_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(75, PIN_75_CONFIG, PIN_75_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 75*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(75, BSP_GPIO_ENABLE_PULLUP_ON_PIN75);
}




/*
 * Configuration function for GPIO pin 76.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort76(void)
{
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 76 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 76 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_76_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 76 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_CTL_IO_0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_76_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_2_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_76_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_76_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN76 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_76_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_76_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN76 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_76_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_76_VALUE         (1)

    #else

        #define PIN_76_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_76_VALUE         (0)

    #endif
#else
    #define PIN_76_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(76, PIN_76_CONFIG, PIN_76_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 76*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(76, BSP_GPIO_ENABLE_PULLUP_ON_PIN76);
}




/*
 * Configuration function for GPIO pin 77.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort77(void)
{
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 77 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 77 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_77_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 77 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_CTL_IO_1 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_77_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_3_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_77_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_77_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN77 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_77_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_77_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN77 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_77_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_77_VALUE         (1)

    #else

        #define PIN_77_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_77_VALUE         (0)

    #endif
#else
    #define PIN_77_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(77, PIN_77_CONFIG, PIN_77_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 77*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(77, BSP_GPIO_ENABLE_PULLUP_ON_PIN77);
}




/*
 * Configuration function for GPIO pin 78.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort78(void)
{
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 78 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 78 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_78_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 78 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_CTL_IO_2 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_78_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_4_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_78_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_78_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN78 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_78_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_78_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN78 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_78_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_78_VALUE         (1)

    #else

        #define PIN_78_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_78_VALUE         (0)

    #endif
#else
    #define PIN_78_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(78, PIN_78_CONFIG, PIN_78_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 78*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(78, BSP_GPIO_ENABLE_PULLUP_ON_PIN78);
}




/*
 * Configuration function for GPIO pin 79.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort79(void)
{
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 79 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 79 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_79_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 79 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_CTL_IO_3 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_79_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_5_INPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_79_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_79_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN79 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_79_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_79_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN79 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_79_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_79_VALUE         (1)

    #else

        #define PIN_79_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_79_VALUE         (0)

    #endif
#else
    #define PIN_79_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(79, PIN_79_CONFIG, PIN_79_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 79*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(79, BSP_GPIO_ENABLE_PULLUP_ON_PIN79);
}




/*
 * Configuration function for GPIO pin 80.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort80(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 80 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 80 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_80_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 80 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_80_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_6_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_80_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_80_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN80 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_80_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_80_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN80 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_80_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_80_VALUE         (1)

    #else

        #define PIN_80_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_80_VALUE         (0)

    #endif
#else
    #define PIN_80_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(80, PIN_80_CONFIG, PIN_80_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 80*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(80, BSP_GPIO_ENABLE_PULLUP_ON_PIN80);
}




/*
 * Configuration function for GPIO pin 81.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort81(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 81 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 81 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_81_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 81 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_81_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_7_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_81_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_81_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN81 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_81_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_81_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN81 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_81_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_81_VALUE         (1)

    #else

        #define PIN_81_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_81_VALUE         (0)

    #endif
#else
    #define PIN_81_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(81, PIN_81_CONFIG, PIN_81_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 81*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(81, BSP_GPIO_ENABLE_PULLUP_ON_PIN81);
}




/*
 * Configuration function for GPIO pin 82.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort82(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 82 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 82 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_82_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 82 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_82_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_8_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_82_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_82_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN82 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_82_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_82_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN82 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_82_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_82_VALUE         (1)

    #else

        #define PIN_82_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_82_VALUE         (0)

    #endif
#else
    #define PIN_82_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(82, PIN_82_CONFIG, PIN_82_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 82*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(82, BSP_GPIO_ENABLE_PULLUP_ON_PIN82);
}




/*
 * Configuration function for GPIO pin 83.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort83(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 83 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 83 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_83_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 83 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_83_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_9_INPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_83_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_83_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN83 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_83_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_83_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN83 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_83_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_83_VALUE         (1)

    #else

        #define PIN_83_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_83_VALUE         (0)

    #endif
#else
    #define PIN_83_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(83, PIN_83_CONFIG, PIN_83_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 83*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(83, BSP_GPIO_ENABLE_PULLUP_ON_PIN83);
}




/*
 * Configuration function for GPIO pin 84.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort84(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 84 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 84 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_84_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 84 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_84_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_0_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_84_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_84_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN84 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_84_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_84_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN84 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_84_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_84_VALUE         (1)

    #else

        #define PIN_84_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_84_VALUE         (0)

    #endif
#else
    #define PIN_84_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(84, PIN_84_CONFIG, PIN_84_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 84*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(84, BSP_GPIO_ENABLE_PULLUP_ON_PIN84);
}




/*
 * Configuration function for GPIO pin 85.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort85(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 85 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 85 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_85_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 85 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_85_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_1_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_85_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_85_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN85 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_85_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_85_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN85 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_85_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_85_VALUE         (1)

    #else

        #define PIN_85_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_85_VALUE         (0)

    #endif
#else
    #define PIN_85_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(85, PIN_85_CONFIG, PIN_85_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 85*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(85, BSP_GPIO_ENABLE_PULLUP_ON_PIN85);
}




/*
 * Configuration function for GPIO pin 86.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort86(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 86 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 86 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_86_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 86 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_86_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_2_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_86_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_86_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN86 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_86_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_86_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN86 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_86_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_86_VALUE         (1)

    #else

        #define PIN_86_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_86_VALUE         (0)

    #endif
#else
    #define PIN_86_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(86, PIN_86_CONFIG, PIN_86_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 86*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(86, BSP_GPIO_ENABLE_PULLUP_ON_PIN86);
}




/*
 * Configuration function for GPIO pin 87.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort87(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 87 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 87 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_87_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 87 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_87_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_3_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_87_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_87_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN87 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_87_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_87_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN87 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_87_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_87_VALUE         (1)

    #else

        #define PIN_87_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_87_VALUE         (0)

    #endif
#else
    #define PIN_87_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(87, PIN_87_CONFIG, PIN_87_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 87*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(87, BSP_GPIO_ENABLE_PULLUP_ON_PIN87);
}




/*
 * Configuration function for GPIO pin 88.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort88(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 88 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 88 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_88_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 88 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_88_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_4_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_88_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_88_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN88 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_88_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_88_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN88 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_88_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_88_VALUE         (1)

    #else

        #define PIN_88_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_88_VALUE         (0)

    #endif
#else
    #define PIN_88_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(88, PIN_88_CONFIG, PIN_88_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 88*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(88, BSP_GPIO_ENABLE_PULLUP_ON_PIN88);
}




/*
 * Configuration function for GPIO pin 89.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort89(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 89 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 89 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_89_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 89 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_89_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_5_OUTPUT == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_89_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_89_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN89 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_89_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_89_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN89 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_89_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_89_VALUE         (1)

    #else

        #define PIN_89_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_89_VALUE         (0)

    #endif
#else
    #define PIN_89_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(89, PIN_89_CONFIG, PIN_89_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 89*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(89, BSP_GPIO_ENABLE_PULLUP_ON_PIN89);
}




/*
 * Configuration function for GPIO pin 90.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort90(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 90 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 90 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_90_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 90 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_90_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_6_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_90_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_90_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN90 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_90_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_90_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN90 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_90_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_90_VALUE         (1)

    #else

        #define PIN_90_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_90_VALUE         (0)

    #endif
#else
    #define PIN_90_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(90, PIN_90_CONFIG, PIN_90_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 90*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(90, BSP_GPIO_ENABLE_PULLUP_ON_PIN90);
}




/*
 * Configuration function for GPIO pin 91.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort91(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 91 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 91 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_91_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 91 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_91_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_7_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_91_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_91_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN91 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_91_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_91_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN91 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_91_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_91_VALUE         (1)

    #else

        #define PIN_91_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_91_VALUE         (0)

    #endif
#else
    #define PIN_91_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(91, PIN_91_CONFIG, PIN_91_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 91*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(91, BSP_GPIO_ENABLE_PULLUP_ON_PIN91);
}




/*
 * Configuration function for GPIO pin 92.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort92(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 92 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 92 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_92_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 92 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_92_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_8_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_92_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_92_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN92 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_92_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_92_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN92 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_92_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_92_VALUE         (1)

    #else

        #define PIN_92_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_92_VALUE         (0)

    #endif
#else
    #define PIN_92_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(92, PIN_92_CONFIG, PIN_92_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 92*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(92, BSP_GPIO_ENABLE_PULLUP_ON_PIN92);
}




/*
 * Configuration function for GPIO pin 93.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort93(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 93 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 93 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_93_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 93 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_93_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_TIMER_9_OUTPUT == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_93_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_93_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN93 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_93_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_93_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN93 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_93_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_93_VALUE         (1)

    #else

        #define PIN_93_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_93_VALUE         (0)

    #endif
#else
    #define PIN_93_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(93, PIN_93_CONFIG, PIN_93_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 93*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(93, BSP_GPIO_ENABLE_PULLUP_ON_PIN93);
}




/*
 * Configuration function for GPIO pin 94.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort94(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 94 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 94 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_94_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 94 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_94_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_94_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_94_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN94 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_94_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_94_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN94 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_94_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_94_VALUE         (1)

    #else

        #define PIN_94_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_94_VALUE         (0)

    #endif
#else
    #define PIN_94_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(94, PIN_94_CONFIG, PIN_94_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 94*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(94, BSP_GPIO_ENABLE_PULLUP_ON_PIN94);
}




/*
 * Configuration function for GPIO pin 95.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort95(void)
{
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 95 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 95 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_95_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 95 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_16_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_95_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_QDC == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_95_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_95_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN95 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_95_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_95_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN95 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_95_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_95_VALUE         (1)

    #else

        #define PIN_95_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_95_VALUE         (0)

    #endif
#else
    #define PIN_95_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(95, PIN_95_CONFIG, PIN_95_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 95*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(95, BSP_GPIO_ENABLE_PULLUP_ON_PIN95);
}




/*
 * Configuration function for GPIO pin 96.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort96(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 96 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 96 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_96_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 96 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_96_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_0_CAN_RXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_96_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_96_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN96 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_96_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_96_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN96 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_96_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_96_VALUE         (1)

    #else

        #define PIN_96_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_96_VALUE         (0)

    #endif
#else
    #define PIN_96_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(96, PIN_96_CONFIG, PIN_96_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 96*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(96, BSP_GPIO_ENABLE_PULLUP_ON_PIN96);
}




/*
 * Configuration function for GPIO pin 97.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort97(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 97 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 97 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_97_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin 97 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_97_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_0_CAN_TXD == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_97_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_97_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN97 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_97_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_97_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN97 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_97_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_97_VALUE         (1)

    #else

        #define PIN_97_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_97_VALUE         (0)

    #endif
#else
    #define PIN_97_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(97, PIN_97_CONFIG, PIN_97_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 97*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(97, BSP_GPIO_ENABLE_PULLUP_ON_PIN97);
}




/*
 * Configuration function for GPIO pin 98.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort98(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 98 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 98 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_98_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 98 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_98_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_RXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_98_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_98_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN98 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_98_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_98_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN98 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_98_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_98_VALUE         (1)

    #else

        #define PIN_98_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_98_VALUE         (0)

    #endif
#else
    #define PIN_98_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(98, PIN_98_CONFIG, PIN_98_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 98*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(98, BSP_GPIO_ENABLE_PULLUP_ON_PIN98);
}




/*
 * Configuration function for GPIO pin 99.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort99(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 99 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 99 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_99_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 99 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_99_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IOP_1_CAN_TXD == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_99_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_99_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN99 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_99_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_99_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN99 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_99_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_99_VALUE         (1)

    #else

        #define PIN_99_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_99_VALUE         (0)

    #endif
#else
    #define PIN_99_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(99, PIN_99_CONFIG, PIN_99_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 99*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(99, BSP_GPIO_ENABLE_PULLUP_ON_PIN99);
}




/*
 * Configuration function for GPIO pin 100.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort100(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 100 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 100 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_100_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_PWM_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 100 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_PWM_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_100_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_PWM_4 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_100_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_100_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN100 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_100_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_100_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN100 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_100_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_100_VALUE         (1)

    #else

        #define PIN_100_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_100_VALUE         (0)

    #endif
#else
    #define PIN_100_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(100, PIN_100_CONFIG, PIN_100_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 100*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(100, BSP_GPIO_ENABLE_PULLUP_ON_PIN100);
}




/*
 * Configuration function for GPIO pin 101.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort101(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 101 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin 101 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_101_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin 101 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_101_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_3 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH)
    #define PIN_101_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_101_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN101 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_101_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_101_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN101 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_101_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_101_VALUE         (1)

    #else

        #define PIN_101_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_101_VALUE         (0)

    #endif
#else
    #define PIN_101_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(101, PIN_101_CONFIG, PIN_101_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 101*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(101, BSP_GPIO_ENABLE_PULLUP_ON_PIN101);
}




/*
 * Configuration function for GPIO pin 102.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort102(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 102 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 102 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_102_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 102 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_102_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_102_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_102_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN102 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_102_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_102_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN102 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_102_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_102_VALUE         (1)

    #else

        #define PIN_102_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_102_VALUE         (0)

    #endif
#else
    #define PIN_102_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(102, PIN_102_CONFIG, PIN_102_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 102*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(102, BSP_GPIO_ENABLE_PULLUP_ON_PIN102);
}




/*
 * Configuration function for GPIO pin 103.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPort103(void)
{
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin 103 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 103 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_0_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_103_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH))
    #error GPIO pin 103 has been configured for more than one function.
    #error (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_IOP_1_24_BIT_BUS == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_103_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_2ND_ALTERNATE_PATH)
    #define PIN_103_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_103_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PIN103 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_103_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_103_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PIN103 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_103_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_103_VALUE         (1)

    #else

        #define PIN_103_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_103_VALUE         (0)

    #endif
#else
    #define PIN_103_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(103, PIN_103_CONFIG, PIN_103_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin 103*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(103, BSP_GPIO_ENABLE_PULLUP_ON_PIN103);
}


#endif /*NS9215*/


/*
 * Configuration function for GPIO pin A24.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPortA24(void)
{
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin A24 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A24 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_24_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    #define PIN_A24_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) && (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A24 has been configured for more than one function.
    #error (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) and (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_A24_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_0 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH)
    #define PIN_A24_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_A24_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PINA24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_A24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A24_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PINA24 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_A24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A24_VALUE         (1)

    #else

        #define PIN_A24_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_A24_VALUE         (0)

    #endif
#else
    #define PIN_A24_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(104, PIN_A24_CONFIG, PIN_A24_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin A24*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(104, BSP_GPIO_ENABLE_PULLUP_ON_PINA24);
}




/*
 * Configuration function for GPIO pin A25.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPortA25(void)
{
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER)  && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH))
    #error GPIO pin A25 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER)  && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER)  && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A25 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER)  && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_25_AND_LOWER)  && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    #define PIN_A25_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) && (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A25 has been configured for more than one function.
    #error (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH) and (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_I2C == BSP_GPIO_MUX_USE_ALTERNATE_PATH)
    #define PIN_A25_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_1 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH)
    #define PIN_A25_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_A25_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PINA25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_A25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A25_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PINA25 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_A25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A25_VALUE         (1)

    #else

        #define PIN_A25_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_A25_VALUE         (0)

    #endif
#else
    #define PIN_A25_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(105, PIN_A25_CONFIG, PIN_A25_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin A25*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(105, BSP_GPIO_ENABLE_PULLUP_ON_PINA25);
}




/*
 * Configuration function for GPIO pin A26.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPortA26(void)
{
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin A26 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A26 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING >= BSP_GPIO_MUX_USE_ADDR_BIT_26_AND_LOWER) && (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING <= BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)
    #define PIN_A26_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH))
    #error GPIO pin A26 has been configured for more than one function.
    #error (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_A26_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_IRQ_2 == BSP_GPIO_MUX_USE_3RD_ALTERNATE_PATH)
    #define PIN_A26_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_A26_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PINA26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_A26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A26_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PINA26 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_A26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A26_VALUE         (1)

    #else

        #define PIN_A26_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_A26_VALUE         (0)

    #endif
#else
    #define PIN_A26_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(106, PIN_A26_CONFIG, PIN_A26_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin A26*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(106, BSP_GPIO_ENABLE_PULLUP_ON_PINA26);
}




/*
 * Configuration function for GPIO pin A27.
 * This function configures the GPIO pin according to values 
 * set in the platform's gpio.h file.
 *
 * This code was generated by the utility program gpioCodeGenerator.
 * Do not modify this code directly.  Modify the input files to gpioCodeGenerator
 * instead and regenerate this code.
 */
static void naSetupPortA27(void)
{
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)  && (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin A27 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)  and (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if ((BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)  && (BSP_GPIO_MUX_UART_REFENCE_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin A27 has been configured for more than one function.
    #error (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER)  and (BSP_GPIO_MUX_UART_REFENCE_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_HIGH_ORDER_ADDRESSING == BSP_GPIO_MUX_USE_ADDR_BIT_27_AND_LOWER) 
    #define PIN_A27_CONFIG    NA_GPIO_SELECT_MUX_FUNC0
#endif

#if ((BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) && (BSP_GPIO_MUX_UART_REFENCE_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH))
    #error GPIO pin A27 has been configured for more than one function.
    #error (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH) and (BSP_GPIO_MUX_UART_REFENCE_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH) cannot both be true in gpio.h.
#endif
#if (BSP_GPIO_MUX_CS0 == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_A27_CONFIG    NA_GPIO_SELECT_MUX_FUNC1
#endif

#if (BSP_GPIO_MUX_UART_REFENCE_CLOCK == BSP_GPIO_MUX_USE_PRIMARY_PATH)
    #define PIN_A27_CONFIG    NA_GPIO_SELECT_MUX_FUNC2
#endif

#if !defined(PIN_A27_CONFIG)
    #if (BSP_GPIO_INITIAL_STATE_PINA27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC0)

        #define PIN_A27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A27_VALUE         (0)

    #elif (BSP_GPIO_INITIAL_STATE_PINA27 == BSP_GPIO_INITIAL_STATE_OUTPUT_DRIVER_LOGIC1)

        #define PIN_A27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_OUTPUT
        #define PIN_A27_VALUE         (1)

    #else

        #define PIN_A27_CONFIG        NA_GPIO_SELECT_MUX_GPIO_INPUT
        #define PIN_A27_VALUE         (0)

    #endif
#else
    #define PIN_A27_VALUE         (0)
#endif

    if (NAconfigureGPIOpin(107, PIN_A27_CONFIG, PIN_A27_VALUE) != SUCCESS)
    {
        /*Error configuring GPIO pin A27*/
        customizeErrorHandler(ERROR_GPIO_MUX, ERROR_GPIO_ERROR_CONFIGURING);
    }
    NAconfigureGPIOpinPullup(107, BSP_GPIO_ENABLE_PULLUP_ON_PINA27);
}




/*
 *
 *  Function: int NAsetGPIOpin(unsigned int channel, unsigned int state)
 *
 *  Description:
 *
 *      This function sets the output level of a output GPIO pin.  The function 
 *      requires the pin to be configured as a GPIO output.
 *
 *  Parameters:
 *
 *      channel         The GPIO channel.  Should be 0 through 72.  The pin
 *                      must also be setup as a GPIO output.
 *      state           Logic level to set the pit.  Non-zero value sets the
 *                      pin to a logic one, otherwise the pin is set to zero.
 *
 *  Return Values:
 *
 *      0                       SUCCESS
 *      NA_GPIO_INVALID_PIN     Invalid channel
 *      NA_GPIO_INVALID_SETUP   Valid channel, but not setup as a GPIO output
 *
 */
int NAsetGPIOpin(unsigned int channel, unsigned int state)
{
    unsigned int subIndex;
    unsigned long volatile *address;
    int irq_state;

    if (!VALID_GPIO_PIN(channel))
    {
        return NA_GPIO_INVALID_PIN;
    }

    subIndex = NA_GPIO_CONFIG_getSubindex(channel);
    address = NA_GPIO_CONFIG_getAddress(channel);
    if (   (narm_read_reg(NA_GPIO_CONFIG, address, func(subIndex)) != NA_GPIO_CONFIG_FUNCTION_GPIO)
        || (narm_read_reg(NA_GPIO_CONFIG, address, dir(subIndex)) != NA_GPIO_CONFIG_OUTPUT))
    {
        return NA_GPIO_INVALID_SETUP;
    }
    
    subIndex = NA_GPIO_CONTROL_getSubindex(channel);
    address = NA_GPIO_CONTROL_getAddress(channel);
    
    irq_state = NASetIRQ (IRQ_DISABLED);
    narm_write_reg(NA_GPIO_CONTROL, address, pin(subIndex), (state != 0));
    NASetIRQ (irq_state);
    
    return 0;
}




/*
 *
 *  Function: int NAgetGPIOpin(unsigned int channel, unsigned int *state)
 *
 *  Description:
 *
 *      This function gets the present level of a GPIO input pin.  The function 
 *      requires the pin to be configured as a GPIO input.
 *
 *  Parameters:
 *
 *      channel         The GPIO channel.  Should be 0 through 49.  
 *      state           pointer to an integer that can be loaded with the 
 *                      logic level of the pin.  Non-zero value represents a 
 *                      logic one, otherwise the pin is zero.
 *
 *  Return Values:
 *
 *      0                       SUCCESS
 *      NA_GPIO_INVALID_PIN     Invalid channel
 *      NA_GPIO_INVALID_SETUP   Valid channel, but not setup as a GPIO output
 *      NA_GPIO_INVALID_ARG     Null state pointer
 *
 */
int NAgetGPIOpin(unsigned int channel, unsigned int *state)
{
    unsigned int subIndex;
    unsigned long volatile *address;
    
    if (!VALID_GPIO_PIN(channel))
    {
        return NA_GPIO_INVALID_PIN;
    }

#ifndef DOTNETMF
#if 0 /* GPIO status register can always be read regardless what function is that GPIO pin was configured */
    subIndex = NA_GPIO_CONFIG_getSubindex(channel);
    address = NA_GPIO_CONFIG_getAddress(channel);
    if (   (narm_read_reg(NA_GPIO_CONFIG, address, func(subIndex)) != NA_GPIO_CONFIG_FUNCTION_GPIO)
        /* || (narm_read_reg(NA_GPIO_CONFIG, address, dir(subIndex)) != NA_GPIO_CONFIG_INPUT) Even GPIO out put pin should be readable */)
    {
        return NA_GPIO_INVALID_SETUP;
    }
#endif
#endif
    
    subIndex = NA_GPIO_STATUS_getSubindex(channel);
    address = NA_GPIO_STATUS_getAddress(channel);
    
    *state = narm_read_reg(NA_GPIO_STATUS, address, pin(subIndex));

    return 0;
}

/*ad*
 * Configures a GPIO multiplexer pin to a GPIO or multiplexer function.
 *
 * @param channel               The GPIO multiplexer channel to be configured
 * @param state                 The configuration for the GPIO multiplexer channel
 * @param initialValue          The initial value for the GPIO pin
 *
 * @return SUCCESS              The GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 * @return NA_GPIO_INVALID_ARG  Invalid state argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAgetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAsetGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 6.2
*ad*/

int NAconfigureGPIOpin(unsigned int channel, unsigned int state, unsigned int initialValue)
{
    unsigned long volatile *address;
    unsigned subIndex;
    int irq_state;
    int retcode = SUCCESS;

    if (!VALID_GPIO_PIN(channel))
    {
        return NA_GPIO_INVALID_PIN;
    }
    
    if ((state & ~NA_GPIO_CONFIG_MASK) != 0)
    {
        return NA_GPIO_INVALID_ARG;
    }
    
    if (state == NA_GPIO_OUTPUT_STATE)
    {
        subIndex = NA_GPIO_CONTROL_getSubindex(channel);
        address = NA_GPIO_CONTROL_getAddress(channel);
    
        irq_state = NASetIRQ (IRQ_DISABLED);
        narm_write_reg(NA_GPIO_CONTROL, address, pin(subIndex), (initialValue != 0));
        NASetIRQ (irq_state);
    }
    
    subIndex = NA_GPIO_CONFIG_getSubindex(channel);
    address = NA_GPIO_CONFIG_getAddress(channel);
    
    switch (state)
    {
        case NA_GPIO_OUTPUT_STATE:
            narm_write_reg(NA_GPIO_CONFIG, address, dir(subIndex), NA_GPIO_CONFIG_OUTPUT);
            narm_write_reg(NA_GPIO_CONFIG, address, func(subIndex), NA_GPIO_CONFIG_FUNCTION_GPIO);
            break;
        case NA_GPIO_INPUT_STATE:
            narm_write_reg(NA_GPIO_CONFIG, address, dir(subIndex), NA_GPIO_CONFIG_INPUT);
            narm_write_reg(NA_GPIO_CONFIG, address, func(subIndex), NA_GPIO_CONFIG_FUNCTION_GPIO);
            break;
        default:
            narm_write_reg(NA_GPIO_CONFIG, address, allConfigBits(subIndex), state);
    }
    return retcode;    
}



/*ad*
 * Configures the internal pullup resister on a GPIO pin.
 *
 * @param channel               The GPIO multiplexer channel to be configured
 * @param isPullupEnabled       set if the pullup should be enabled
 *
 * @return SUCCESS              The GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAgetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAsetGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 6.2
*ad*/

int NAconfigureGPIOpinPullup(unsigned int channel, unsigned int isPullupEnabled)
{
    unsigned long volatile *address;
    unsigned subIndex;
    int retcode = SUCCESS;

    if (!VALID_GPIO_PIN(channel))
    {
        return NA_GPIO_INVALID_PIN;
    }
    
    
    subIndex = NA_GPIO_CONFIG_getSubindex(channel);
    address = NA_GPIO_CONFIG_getAddress(channel);
    
    if (isPullupEnabled)
    {
        narm_write_reg(NA_GPIO_CONFIG, address, puen(subIndex), NA_GPIO_CONFIG_ENABLE_PULLUP);
    }
    else
    {
        narm_write_reg(NA_GPIO_CONFIG, address, puen(subIndex), NA_GPIO_CONFIG_DISABLE_PULLUP);
    }
        
    return retcode;    
}

                    
/*
 * To get the a GPIO multiplexer configuration value of the specified pin.
 *
 * @param channel         The GPIO multiplexer pin number
 * @param state           To store the GPIO multiplexer pin configuration value 
 *
 * @return SUCCESS              The GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAgetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAsetGPIOpin
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:NAconfigureGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 7.1
 */

int NAgetGPIOpinConfig(unsigned int channel, unsigned int *state)
{
    unsigned long volatile *address;
    unsigned subIndex;
    int retcode = SUCCESS;

    if (!VALID_GPIO_PIN(channel))
    {
        retcode = NA_GPIO_INVALID_PIN;
    }
    else
    {
        subIndex = NA_GPIO_CONFIG_getSubindex(channel);
        address = NA_GPIO_CONFIG_getAddress(channel);
    
        *state = narm_read_reg(NA_GPIO_CONFIG, address, allConfigBits(subIndex));
    }
    
    return retcode;    
}


/*ad*
 * Inverts a GPIO multiplexer pin.
 *
 * @param channel    The GPIO multiplexer channel to be configured
 * @param enable     Inverts if this value is TRUE; otherwise, sets to normal
 *
 * @return SUCCESS              GPIO MUX pin has configured successfully.
 * @return NA_GPIO_INVALID_PIN  Invalid channel argument.
 *
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Pin_States 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface:GPIO_Return_Values 
 * @see @link BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360:NAconfigureGPIOpin
 *
 * @external
 * @include "gpiomux_def.h"
 * @category BSP:Device_Drivers:Mux_and_GPIO:GPIOInterface9360
 * @since 6.2
*ad*/
int NAInvertGPIOpin(unsigned int channel, BOOLEAN enable)
{
    unsigned long volatile *address;
    unsigned subIndex;
    int retcode = SUCCESS;

    if (!VALID_GPIO_PIN(channel))
    {
        retcode = NA_GPIO_INVALID_PIN;
    }
    else
    {
        subIndex = NA_GPIO_CONFIG_getSubindex(channel);
        address = NA_GPIO_CONFIG_getAddress(channel);
    
        narm_write_reg(NA_GPIO_CONFIG, address, inv(subIndex), enable);
    }
    
    return retcode;    
}

#ifdef BSP_GPIO_BOOT_MUX_PIN
static void NAToggleBootMux(unsigned int channel)
{
    if (NAconfigureGPIOpin(channel, NA_GPIO_OUTPUT_STATE, 0) != SUCCESS)
        return;      
    NAsetGPIOpin(channel, 1);
    NAsetGPIOpin(channel, 0);     
}
#endif

void NAConfigureGpioInterrupt(int ext_int_index, int lowPolarity, int edgeSensitive)
{
	narm_write_reg (NA_SCM_EICR_REG, NA_SCM_EICR_GET_ADDRESS(ext_int_index), plty, lowPolarity);
    narm_write_reg (NA_SCM_EICR_REG, NA_SCM_EICR_GET_ADDRESS(ext_int_index), lvedg, edgeSensitive);
}

void NAClearExtInterrupt(int ext_int_index)
{
    narm_write_reg (NA_SCM_EICR_REG, NA_SCM_EICR_GET_ADDRESS(ext_int_index), clr, 1);
    narm_write_reg (NA_SCM_EICR_REG, NA_SCM_EICR_GET_ADDRESS(ext_int_index), clr, 0);
}

/*
 *
 *  Function: void customizeSetupGPIO(int chipType, int softRestart)
 *
 *  Description:
 *
 *      This function sets up all GPIO pins.  This version of the function 
 *      sets up the ports to support a NET+OS development board.  The port 
 *      is disabled (left configured for GPIO input) if the port is not used.
 *
 *  Notes:
 *
 *      THIS CODE MUST BE CUSTOMIZED FOR YOUR BOARD.  The implementation
 *      is for NET+OS development boards only.
 *
 *      This function will be called by ncc_init() before RAM has been
 *      configured.  The program stack is thus particularly small, so 
 *      limit the amount of local scope.   Also do not use global or static 
 *      variables.
 *
 *  Parameters:
 *
 *      chipType        set to indicate the base chip type of the processor.
 *                      The subrevision information has been masked off.
 *      softRestart     set if a software restart has taken place.
 *
 *
 *  Return Values:
 *
 *      none
 *
 */
void customizeSetupGPIO(int chipType, int softRestart)
{
    NA_UNUSED_PARAM(chipType);
    NA_UNUSED_PARAM(softRestart);

    #ifdef BSP_GPIO_BOOT_MUX_PIN
    NAToggleBootMux(BSP_GPIO_BOOT_MUX_PIN);
    #endif

    naSetupPort0();
    naSetupPort1();
    naSetupPort2();
    naSetupPort3();
    naSetupPort4();
    naSetupPort5();
    naSetupPort6();
    naSetupPort7();
    naSetupPort8();
    naSetupPort9();
    naSetupPort10();
    naSetupPort11();
    naSetupPort12();
    naSetupPort13();
    naSetupPort14();
    naSetupPort15();
    naSetupPort16();
    naSetupPort17();
    naSetupPort18();
    naSetupPort19();
    naSetupPort20();
    naSetupPort21();
    naSetupPort22();
    naSetupPort23();
    naSetupPort24();
    naSetupPort25();
    naSetupPort26();
    naSetupPort27();
    naSetupPort28();
    naSetupPort29();
    naSetupPort30();
    naSetupPort31();
    naSetupPort32();
    naSetupPort33();
    naSetupPort35();
    naSetupPort34();
    naSetupPort36();
    naSetupPort37();
    naSetupPort38();
    naSetupPort39();
    naSetupPort40();
    naSetupPort41();
    naSetupPort42();
    naSetupPort43();
    naSetupPort45();
    naSetupPort44();
    naSetupPort46();
    naSetupPort47();
    naSetupPort48();
    naSetupPort49();
/*
 * NS9210 and NS9215 are the same processor in different packages.  The NS9215 has more pins.  
 * The first 50 GPIO pins are the same on both chips.  The NS9215 has 54 more though.
 */
#ifdef NS9215
    naSetupPort50();
    naSetupPort51();
    naSetupPort52();
    naSetupPort53();
    naSetupPort55();
    naSetupPort54();
    naSetupPort56();
    naSetupPort57();
    naSetupPort58();
    naSetupPort59();
    naSetupPort60();
    naSetupPort61();
    naSetupPort62();
    naSetupPort63();
    naSetupPort64();
    naSetupPort65();
    naSetupPort66();
    naSetupPort67();
    naSetupPort68();
    naSetupPort69();
    naSetupPort70();
    naSetupPort71();
    naSetupPort72();
    naSetupPort73();
    naSetupPort74();
    naSetupPort75();
    naSetupPort76();
    naSetupPort77();
    naSetupPort78();
    naSetupPort79();
    naSetupPort80();
    naSetupPort81();
    naSetupPort82();
    naSetupPort83();
    naSetupPort84();
    naSetupPort85();
    naSetupPort86();
    naSetupPort87();
    naSetupPort88();
    naSetupPort89();
    naSetupPort90();
    naSetupPort91();
    naSetupPort92();
    naSetupPort93();
    naSetupPort94();
    naSetupPort95();
    naSetupPort96();
    naSetupPort97();
    naSetupPort98();
    naSetupPort99();
    naSetupPort100();
    naSetupPort101();
    naSetupPort102();
    naSetupPort103();
#endif

    naSetupPortA24();
    naSetupPortA25();
    naSetupPortA26();
    naSetupPortA27();
    
}



